@@ -1,23 +1,9 @@
-//! The builder design in this module
-//! supports code generation to the `abstraps` IR.
-//!
-//! The interfaces provided below allow customized code generation
-//! for user-defined intrinsics and lowering.
-
 use crate::core::diagnostics::LocationInfo;
 use crate::core::ir::{Attribute, BasicBlock, Intrinsic, Operation, SupportsInterfaceTraits, Var};
 use crate::core::region::Region;
-use alloc::string::String;
-use alloc::vec::Vec;
 use color_eyre::Report;
 use std::collections::HashMap;
 
-#[derive(Debug)]
-pub enum BuilderError {
-    BuilderCreationFailure,
-    Caseless,
-}
-
 #[derive(Debug)]
 pub struct OperationBuilder {
     latest: Vec<Var>,
@@ -27,7 +13,7 @@ pub struct OperationBuilder {
     operands: Vec<Var>,
     attributes: HashMap<String, Box<dyn Attribute>>,
     regions: Vec<Region>,
-    successors: Vec<BasicBlock>,
+    successors: Vec<usize>,
 }
 
 impl SupportsInterfaceTraits for OperationBuilder {
@@ -35,6 +21,10 @@ impl SupportsInterfaceTraits for OperationBuilder {
         &self.intrinsic
     }
 
+    fn get_operands(&self) -> &[Var] {
+        &self.operands
+    }
+
     fn get_regions(&self) -> &[Region] {
         &self.regions
     }
@@ -86,6 +76,10 @@ impl OperationBuilder {
         self.operands.to_vec()
     }
 
+    pub fn set_successors(&mut self, blks: Vec<usize>) {
+        self.successors = blks;
+    }
+
     pub fn set_cursor(&mut self, reg: usize, blk: usize) {
         self.cursor = (reg, blk);
     }
