@@ -1,4 +1,5 @@
 use color_eyre::Report;
+use yansi::Paint;
 
 /// Diagnostics setup for tracing and error reporting.
 pub fn diagnostics_setup() -> Result<(), Report> {
@@ -11,10 +12,14 @@ pub fn diagnostics_setup() -> Result<(), Report> {
         std::env::set_var("RUST_LOG", "info")
     }
     tracing_subscriber::fmt::fmt().init();
-
     Ok(())
 }
 
+/// Globally disable terminal color printing.
+pub fn diagnostics_color_disable() {
+    Paint::disable();
+}
+
 /////
 ///// Locations
 /////
