@@ -44,11 +44,12 @@ macro_rules! vtable_for {
         let x = ::std::ptr::null::<$x>() as *const $y;
         #[allow(unused_unsafe)]
         unsafe {
-            ::std::mem::transmute::<_, $crate::TraitObject>(x).vtable
+            ::std::mem::transmute::<_, $crate::core::TraitObject>(x).vtable
         }
     }};
 }
 
+#[doc(hidden)]
 #[macro_export]
 macro_rules! mopo {
     ($name:ty) => {
@@ -57,7 +58,7 @@ macro_rules! mopo {
                 if let Some(vtable) = self.query_vtable(::std::any::TypeId::of::<U>()) {
                     unsafe {
                         let data = self as *const Self;
-                        let u = $crate::TraitObject {
+                        let u = $crate::core::TraitObject {
                             data: data as *const (),
                             vtable,
                         };
@@ -72,7 +73,7 @@ macro_rules! mopo {
                 if let Some(vtable) = self.query_vtable(::std::any::TypeId::of::<U>()) {
                     unsafe {
                         let data = self as *mut Self;
-                        let mut u = $crate::TraitObject {
+                        let mut u = $crate::core::TraitObject {
                             data: data as *const (),
                             vtable,
                         };
@@ -89,7 +90,7 @@ macro_rules! mopo {
                 if let Some(vtable) = self.query_vtable(::std::any::TypeId::of::<U>()) {
                     unsafe {
                         let data = Box::into_raw(self);
-                        let mut u = $crate::TraitObject {
+                        let mut u = $crate::core::TraitObject {
                             data: data as *const (),
                             vtable,
                         };
@@ -108,7 +109,7 @@ macro_rules! mopo {
                 if let Some(vtable) = self_.query_vtable(::std::any::TypeId::of::<U>()) {
                     unsafe {
                         let data = ::std::sync::Arc::into_raw(self_);
-                        let mut u = $crate::TraitObject {
+                        let mut u = $crate::core::TraitObject {
                             data: data as *const (),
                             vtable,
                         };
@@ -128,7 +129,7 @@ macro_rules! mopo {
                 if let Some(vtable) = self_.query_vtable(::std::any::TypeId::of::<U>()) {
                     unsafe {
                         let data = ::std::rc::Rc::into_raw(self_);
-                        let mut u = $crate::TraitObject {
+                        let mut u = $crate::core::TraitObject {
                             data: data as *const (),
                             vtable,
                         };
@@ -142,7 +143,7 @@ macro_rules! mopo {
                 }
             }
             pub fn obj_partial_eq(&self, other: &Self) -> bool {
-                if let Some(x) = self.query_ref::<dyn $crate::ObjectPartialEq>() {
+                if let Some(x) = self.query_ref::<dyn $crate::core::ObjectPartialEq>() {
                     x.obj_eq(other.query_ref().unwrap())
                 } else {
                     std::ptr::eq(
@@ -152,7 +153,7 @@ macro_rules! mopo {
                 }
             }
             pub fn obj_partial_cmp(&self, other: &Self) -> Option<::std::cmp::Ordering> {
-                if let Some(x) = self.query_ref::<dyn $crate::ObjectPartialOrd>() {
+                if let Some(x) = self.query_ref::<dyn $crate::core::ObjectPartialOrd>() {
                     x.obj_partial_cmp(other.query_ref().unwrap())
                 } else {
                     None
@@ -167,7 +168,7 @@ macro_rules! mopo {
         impl ::std::borrow::ToOwned for $name {
             type Owned = Box<$name>;
             fn to_owned(&self) -> Box<$name> {
-                self.query_ref::<dyn $crate::ObjectClone>()
+                self.query_ref::<dyn $crate::core::ObjectClone>()
                     .expect("Object not clonable!")
                     .obj_clone()
                     .query::<$name>()
@@ -187,7 +188,7 @@ macro_rules! mopo {
             fn eq(&self, other: &Self) -> bool {
                 // Require `Eq` rather than `PartialEq` as this allows `Object`s to be used as
                 // key in hash maps
-                if let Some(x) = self.query_ref::<dyn $crate::ObjectEq>() {
+                if let Some(x) = self.query_ref::<dyn $crate::core::ObjectEq>() {
                     x.obj_eq(other.query_ref().unwrap())
                 } else {
                     // This trivially meets the requirements of `Eq`
@@ -206,7 +207,7 @@ macro_rules! mopo {
         }
         impl ::std::cmp::Ord for $name {
             fn cmp(&self, other: &Self) -> ::std::cmp::Ordering {
-                if let Some(x) = self.query_ref::<dyn $crate::ObjectOrd>() {
+                if let Some(x) = self.query_ref::<dyn $crate::core::ObjectOrd>() {
                     if let Some(o) = x.obj_cmp(other.query_ref().unwrap()) {
                         return o;
                     }
@@ -216,7 +217,7 @@ macro_rules! mopo {
         }
         impl ::std::hash::Hash for $name {
             fn hash<H: ::std::hash::Hasher>(&self, state: &mut H) {
-                if let Some(x) = self.query_ref::<dyn $crate::ObjectHash>() {
+                if let Some(x) = self.query_ref::<dyn $crate::core::ObjectHash>() {
                     x.obj_hash(state)
                 } else {
                     state.write_usize(self as *const Self as *const () as usize)
@@ -226,9 +227,12 @@ macro_rules! mopo {
     };
 }
 
-/// This trait is the primary function of the library. `Object` trait objects
+/// `Object` trait objects
 /// can be freely queried for any other trait, allowing conversion between
 /// trait objects.
+///
+/// This trait is the core provider of the interface functionality. Consumers of the library should
+/// not need to deeply understand `Object`.
 pub unsafe trait Object: Any {
     /// This is implemented by the `interfaces!` macro, and should never be
     /// manually implemented.
@@ -253,6 +257,7 @@ mopo!(dyn Object);
 /// This is an object-safe version of `Clone`, which is automatically
 /// implemented for all `Clone + Object` types. This is a support trait used to
 /// allow `Object` trait objects to be clonable.
+#[doc(hidden)]
 pub trait ObjectClone {
     fn obj_clone(&self) -> Box<dyn Object>;
 }
@@ -265,6 +270,7 @@ impl<T: Clone + Object> ObjectClone for T {
 /// This is an object-safe version of `PartialEq`, which is automatically
 /// implemented for all `PartialEq + Object` types. This is a support trait used to
 /// allow `Object` trait objects to be comparable in this way.
+#[doc(hidden)]
 pub trait ObjectPartialEq {
     fn obj_eq(&self, other: &dyn Object) -> bool;
 }
@@ -281,12 +287,14 @@ impl<T: PartialEq + Object> ObjectPartialEq for T {
 /// This is an object-safe version of `Eq`, which is automatically
 /// implemented for all `Eq + Object` types. This is a support trait used to
 /// allow `Object` trait objects to be comparable in this way.
+#[doc(hidden)]
 pub trait ObjectEq: ObjectPartialEq {}
 impl<T: Eq + Object> ObjectEq for T {}
 
 /// This is an object-safe version of `PartialOrd`, which is automatically
 /// implemented for all `PartialOrd + Object` types. This is a support trait used to
 /// allow `Object` trait objects to be comparable in this way.
+#[doc(hidden)]
 pub trait ObjectPartialOrd {
     fn obj_partial_cmp(&self, other: &dyn Object) -> Option<std::cmp::Ordering>;
 }
@@ -303,6 +311,7 @@ impl<T: PartialOrd + Object> ObjectPartialOrd for T {
 /// This is an object-safe version of `Ord`, which is automatically
 /// implemented for all `Ord + Object` types. This is a support trait used to
 /// allow `Object` trait objects to be comparable in this way.
+#[doc(hidden)]
 pub trait ObjectOrd {
     fn obj_cmp(&self, other: &dyn Object) -> Option<std::cmp::Ordering>;
 }
@@ -318,6 +327,7 @@ impl<T: Ord + Object> ObjectOrd for T {
 ///
 /// Note: `Object`s are not guaranteed to hash to the same value as their
 /// underlying type.
+#[doc(hidden)]
 pub trait ObjectHash {
     fn obj_hash(&self, state: &mut dyn Hasher);
 }
@@ -329,25 +339,30 @@ impl<T: Hash + Object> ObjectHash for T {
     }
 }
 
+/// A macro which exposes static (virtual) lookup for trait objects
+/// which implement the `Object` interface.
+///
+/// This is internal, and provides core functionality for [`intrinsic!`].
+/// Users are not expected to interact with this functionality directly.
 #[macro_export(local_inner_macros)]
 macro_rules! interfaces {
     (@unbracket $(($($v:tt)*))*) => ($($($v)*)*);
     (@inner $imp:tt $cond:tt $name:ty: $($iface:ty),+ {}) => (
-        interfaces!(@unbracket $imp ($crate::HasInterface<$name> for $name) $cond ({}));
-        interfaces!(@unbracket $imp ($crate::HasInterface<dyn $crate::Object> for $name) $cond ({}));
-        $(interfaces!(@unbracket $imp ($crate::HasInterface<$iface> for $name) $cond ({}));)*
-        interfaces!(@unbracket $imp ($crate::Object for $name) $cond ({
-            fn query_vtable(&self, id: ::std::any::TypeId) -> Option<$crate::VTable> {
+        interfaces!(@unbracket $imp ($crate::core::HasInterface<$name> for $name) $cond ({}));
+        interfaces!(@unbracket $imp ($crate::core::HasInterface<dyn $crate::core::Object> for $name) $cond ({}));
+        $(interfaces!(@unbracket $imp ($crate::core::HasInterface<$iface> for $name) $cond ({}));)*
+        interfaces!(@unbracket $imp ($crate::core::Object for $name) $cond ({
+            fn query_vtable(&self, id: ::std::any::TypeId) -> Option<$crate::core::VTable> {
                 if id == ::std::any::TypeId::of::<$name>() {
-                    Some($crate::VTable::none())
-                } else if id == ::std::any::TypeId::of::<dyn $crate::Object>() {
-                    Some(vtable_for!($name as dyn $crate::Object))
+                    Some($crate::core::VTable::none())
+                } else if id == ::std::any::TypeId::of::<dyn $crate::core::Object>() {
+                    Some(vtable_for!($name as dyn $crate::core::Object))
                 } else $(if id == ::std::any::TypeId::of::<$iface>() {
                     Some(vtable_for!($name as $iface))
                 } else)* {
                     // If "dynamic" feature is enabled, fall back to
                     // looking in the registry
-                    { $crate::find_in_registry::<$name>(id) }
+                    { $crate::core::find_in_registry::<$name>(id) }
                 }
             }
         }));
@@ -535,12 +550,18 @@ pub fn find_in_registry<Type: 'static + ?Sized>(trait_id: TypeId) -> Option<VTab
     with_registry(|registry| registry.find::<Type>(trait_id))
 }
 
+/// Inserts an interface into the global crate-managed virtual table
+/// at runtime.
+///
+/// Provides core functionality for the abstract interpretation interface,
+/// and that's the only time the user will be expected to interact
+/// with this functionality.
 #[macro_export]
 macro_rules! dynamic_interfaces {
     ($($name:ty: $($iface:ty),+;)*) => (
-        $crate::with_registry_mut(|registry| { unsafe { $(
-            registry.register::<$name, $name>($crate::VTable::none());
-            registry.register::<$name, dyn $crate::Object>(vtable_for!($name as dyn $crate::Object));
+        $crate::core::with_registry_mut(|registry| { unsafe { $(
+            registry.register::<$name, $name>($crate::core::VTable::none());
+            registry.register::<$name, dyn $crate::core::Object>(vtable_for!($name as dyn $crate::core::Object));
             $(
             registry.register::<$name, $iface>(vtable_for!($name as $iface));
             )+
@@ -680,12 +701,4 @@ mod tests {
         let dyn2: Option<&dyn Dynamic> = x.query_ref();
         assert!(dyn2.unwrap().test() == 42);
     }
-
-    #[test]
-    fn test_primitives() {
-        Box::new(1);
-        Box::new(1f32);
-        Box::new("test".to_string());
-        Box::new(vec![1, 2, 3]);
-    }
 }
