@@ -1,9 +1,11 @@
-use crate::dialects::builtin::attributes::{Symbol, SymbolTable};
-use crate::dialects::builtin::traits::{ProvidesSymbol, ProvidesSymbolTable};
+use crate::core::*;
+use crate::dialects::builtin::*;
 use crate::*;
 
-// Module intrinsic.
-intrinsic!(Module, "builtin", "module", ProvidesSymbolTable);
+intrinsic! {
+    Module: ["builtin", "module"],
+    [ProvidesSymbolTable], extern: []
+}
 
 impl Module {
     pub fn get_builder(&self, name: &str, loc: LocationInfo) -> OperationBuilder {
@@ -13,16 +15,19 @@ impl Module {
         b.push_region(r);
         let blk = BasicBlock::default();
         b.push_block(blk);
-        let st = SymbolTable::new();
+        let st = SymbolTableAttr::new();
         b.insert_attr("symbols", Box::new(st));
-        let sym_name = Symbol::new(name);
+        let sym_name = SymbolAttr::new(name);
         b.insert_attr("symbol", Box::new(sym_name));
         b
     }
 }
 
-// Function intrinsic.
-intrinsic!(Func, "builtin", "func", ProvidesSymbol);
+intrinsic! {
+    Func: ["builtin", "func"],
+    [ProvidesSymbol, ProvidesLinkage, FunctionLike, RequiresTerminators],
+    extern: []
+}
 
 impl Func {
     pub fn get_builder(&self, name: &str, loc: LocationInfo) -> OperationBuilder {
@@ -32,8 +37,10 @@ impl Func {
         b.push_region(r);
         let blk = BasicBlock::default();
         b.push_block(blk);
-        let attr = Symbol::new(name);
+        let attr = SymbolAttr::new(name);
         b.insert_attr("symbol", Box::new(attr));
+        let lattr = LinkageAttr::Private;
+        b.insert_attr("linkage", Box::new(lattr));
         b
     }
 }
