@@ -1,15 +1,11 @@
 #![doc = include_str!("../README.md")]
-#![warn(missing_docs)]
 #![warn(rustdoc::missing_doc_code_examples)]
 
 pub mod backends;
+pub mod core;
 pub mod dialects;
 
-mod core;
-pub use self::core::*;
-
 #[macro_use]
-extern crate alloc;
 extern crate color_eyre;
 extern crate tracing;
 extern crate tracing_subscriber;
