@@ -1,8 +1,9 @@
+use abstraps::core::*;
+use abstraps::dialects::base::*;
 use abstraps::dialects::builtin::*;
-use abstraps::dialects::std::*;
 use abstraps::*;
 
-intrinsic!(Add, "arith", "add");
+intrinsic!(Add: ["arith", "add"], [], extern: []);
 
 impl Add {
     pub fn get_builder(&self, operands: Vec<Var>, loc: LocationInfo) -> OperationBuilder {
@@ -22,7 +23,8 @@ fn passes_0() -> Result<(), Report> {
     let add1 = Add.get_builder(operands, LocationInfo::Unknown);
     let ret = func1.push(add1)?;
     let add2 = Add.get_builder(vec![ret, ret], LocationInfo::Unknown);
-    func1.push(add2)?;
+    let v = func1.push(add2)?;
+    func1.push(Return.get_builder(vec![v], LocationInfo::Unknown))?;
     let mut func2 = Func.get_builder("new_func2", LocationInfo::Unknown);
     let operands = vec![func2.push_arg()?, func2.push_arg()?];
     let add1 = Add.get_builder(operands, LocationInfo::Unknown);
